﻿#include <string.h>
#include "protocol.h"

using namespace std;

/********** 协议基类 **********/
/**
 * 构造方法。
 * @param type  协议类型
 */
Protocol::Protocol(Type type) : m_type(type)
{
}

/**
 * 获取协议类型。
 * @return 协议类型
 */
Protocol::Type Protocol::getType()
{
    return m_type;
}

/********** 登录请求 **********/
/**
 * 构造方法。
 * @param qsUserName    用户名
 * @param qsPassword    密码
 */
LogonRequest::LogonRequest(const QString& qsUserName, const QString& qsPassword) :
    Protocol(Protocol::Logon)
{
    strcpy( m_szUserName, qsUserName.toStdString().c_str() );
    strcpy( m_szPassword, qsPassword.toStdString().c_str() );
}

/**
 * 获取用户名。
 * @return 用户名
 */
const char* LogonRequest::getUserName() const
{
    return m_szUserName;
}

/**
 * 获取密码。
 * @return 密码
 */
const char* LogonRequest::getPassword() const
{
    return m_szPassword;
}

/********** 登录结果 **********/
/**
 * 构造方法。
 */
LogonResult::LogonResult() :
    Protocol(Protocol::Logon),
    m_nFriendsCount(0)
{
    memset( m_nFriendIconIndexs, 0, sizeof(m_nFriendIconIndexs) );
    memset( m_szFriendNames, 0, sizeof(m_szFriendNames) );
}

/**
 * 获取登录结果。
 * @return 登录结果
 */
Result::Code LogonResult::getResultCode() const
{
    return m_resultCode;
}

/**
 * 获取头像索引。
 * @return 头像索引
 */
int LogonResult::getIconIndex() const
{
    return m_nIconIndex;
}

/**
 * 获取好友数。
 * @return 好友数
 */
int LogonResult::getFriendsCount() const
{
    return m_nFriendsCount;
}

/**
 * 获取好友名列表。
 * @param friendNames   好友名列表
 */
void LogonResult::getFriendNames(QStringList& friendNames) const
{
    QByteArray byteArray( m_szFriendNames, sizeof(m_szFriendNames) );
    QList<QByteArray> friendsNames = byteArray.split(Const::SPLIT_FLAT);

    int nCount = friendsNames.count();
    for (int i = 0; i < nCount; i++)
    {
        friendNames.append(QString(friendsNames[i]));
    }
}

/**
 * 获取好友头像索引列表。
 * @param iconIndexsList    好友头像索引列表
 */
void LogonResult::getFriendIconIndexs(QList<int>& iconIndexs) const
{
    for (int i = 0; i < m_nFriendsCount; i++)
    {
        iconIndexs.append(m_nFriendIconIndexs[i]);
    }
}

/**
 * 获取某个好友是否在线。
 * @param nIndex    好友索引
 * @return true 表示在线，false 表示不在线
 */
bool LogonResult::isFriendOnline(int nIndex) const
{
    return m_bFriendsOnline[nIndex];
}

/**
 * 设置登录结果。
 * @param resultCode    登录结果
 */
void LogonResult::setErrorCode(Result::Code resultCode)
{
    m_resultCode = resultCode;
}

/**
 * 设置头像索引。
 * @param nIconIndex    头像索引
 */
void LogonResult::setIconIndex(int nIconIndex)
{
    m_nIconIndex = nIconIndex;
}

/**
 * 设置好友数。
 * @param nFriendsCount 好友数
 */
void LogonResult::setFriendsCount(int nFriendsCount)
{
    m_nFriendsCount = nFriendsCount;
}

/**
 * 设置好友名列表。
 * @param qsFriendNames 好友名列表
 */
void LogonResult::setFriendNames(const QString& qsFriendNames)
{
    strcpy( m_szFriendNames, qsFriendNames.toStdString().c_str() );
}

/**
 * 设置好友头像索引列表。
 * @param nIconIndexs   头像索引列表
 * @param nCount        列表中的元素个数
 */
void LogonResult::setFriendIconIndexs(const int* nIconIndexs, int nCount)
{
    memcpy( m_nFriendIconIndexs, nIconIndexs, nCount * sizeof(int) );
}

/**
 * 设置某个好友的在线状态。
 * @param nIndex    好友索引
 * @param bOnLine   true 表示在线，false 表示不在线
 */
void LogonResult::setFriendsOnline(int nIndex, bool bOnLine)
{
    m_bFriendsOnline[nIndex] = bOnLine;
}

/********** 注册请求 **********/
/**
 * 构造方法。
 * @param qsUserName    用户名
 * @param qsPassword    密码
 * @param nIconIndex    头像索引
 */
RegRequest::RegRequest(const QString& qsUserName, const QString& qsPassword, int nIconIndex) :
    Protocol(Protocol::Reg),
    m_nIconIndex(nIconIndex)
{
    strcpy( m_szUserName, qsUserName.toStdString().c_str() );
    strcpy( m_szPassword, qsPassword.toStdString().c_str() );
}

/**
 * 获取用户名。
 * @return 用户名
 */
const char* RegRequest::getUserName() const
{
    return m_szUserName;
}

/**
 * 获取密码。
 * @return 密码
 */
const char* RegRequest::getPassword() const
{
    return m_szPassword;
}

/**
 * 获取头像索引。
 * @return 头像索引
 */
int RegRequest::getIconIndex() const
{
    return m_nIconIndex;
}

/********** 注册结果 **********/
/**
 * 构造方法。
 */
RegResult::RegResult() : Protocol(Protocol::Reg)
{
}

/**
 * 获取结果。
 * @return 结果
 */
Result::Code RegResult::getResultCode() const
{
    return m_resultCode;
}

/**
 * 设置结果。
 * @param resultCode    结果
 */
void RegResult::setErrorCode(Result::Code resultCode)
{
    m_resultCode = resultCode;
}

/********** 添加好友请求 **********/
/**
 * 构造方法。
 * @param qsSrcUserName 源用户名
 * @param qsDstUserName 目标用户名
 * @param nIconIndex    源用户的头像索引
 */
AddFriendRequest::AddFriendRequest(const QString& qsSrcUserName,
    const QString& qsDstUserName, int nIconIndex) :
    Protocol(Protocol::AddFriendRequest),
    m_nIconIndex(nIconIndex)
{
    strcpy( m_szSrcUserName, qsSrcUserName.toStdString().c_str() );
    strcpy( m_szDstUserName, qsDstUserName.toStdString().c_str() );
}

/**
 * 获取源用户名。
 * @return 源用户名
 */
const char* AddFriendRequest::getSrcUserName() const
{
    return m_szSrcUserName;
}

/**
 * 获取目标用户名。
 * @return 目标用户名
 */
const char* AddFriendRequest::getDstUserName() const
{
    return m_szDstUserName;
}

/**
 * 获取源用户头像索引。
 * @return 源用户头像索引
 */
int AddFriendRequest::getIconIndex() const
{
    return m_nIconIndex;
}

/********** 添加好友结果 **********/
/**
 * 构造方法。
 * @param resultCode    结果
 * @param qsSrcUserName 源用户名
 * @param qsDstUserName 目标用户名
 * @param nIconIndex    目标用户头像索引
 */
AddFriendResult::AddFriendResult(Result::Code resultCode,
    const QString& qsSrcUserName, const QString& qsDstUserName, int nIconIndex) :
    Protocol(Protocol::AddFriendResult),
    m_resultCode(resultCode),
    m_nDstUserIconIndex(nIconIndex)
{
    if ( !qsSrcUserName.isEmpty() )
    {
        strcpy( m_szSrcUserName, qsSrcUserName.toStdString().c_str() );
    }
    else
    {
        memset( m_szSrcUserName, 0, sizeof(m_szSrcUserName) );
    }

    if ( !qsDstUserName.isEmpty() )
    {
        strcpy( m_szDstUserName, qsDstUserName.toStdString().c_str() );
    }
    else
    {
        memset( m_szDstUserName, 0, sizeof(m_szDstUserName) );
    }
}

/**
 * 获取结果。
 * @return 结果
 */
Result::Code AddFriendResult::getResultCode() const
{
    return m_resultCode;
}

/**
 * 获取源用户名。
 * @return 源用户名
 */
const char* AddFriendResult::getSrcUserName() const
{
    return m_szSrcUserName;
}

/**
 * 获取目标用户名。
 * @return 目标用户名
 */
const char* AddFriendResult::getDstUserName() const
{
    return m_szDstUserName;
}

/**
 * 获取目标用户头像索引。
 * @return 目标用户头像索引
 */
int AddFriendResult::getIconIndex() const
{
    return m_nDstUserIconIndex;
}

/********** 客户端向服务器发送的聊天信息 **********/
/**
 * 构造方法。
 * @param qsSrcUserName 源用户名
 * @param qsDstUserName 目标用户名
 * @param qsContent     聊天内容
 */
Chat_C2S::Chat_C2S(const QString& qsSrcUserName,
    const QString& qsDstUserName, const QString& qsContent) :
    Protocol(Protocol::Chat)
{
    strcpy( m_szSrcUserName, qsSrcUserName.toStdString().c_str() );
    strcpy( m_szDstUserName, qsDstUserName.toStdString().c_str() );
    strcpy( m_szContent, qsContent.toStdString().c_str() );
}

/**
 * 获取源用户名。
 * @return 源用户名
 */
const char* Chat_C2S::getSrcUserName() const
{
    return m_szSrcUserName;
}

/**
 * 获取目标用户名。
 * @return 目标用户名
 */
const char* Chat_C2S::getDstUserName() const
{
    return m_szDstUserName;
}

/**
 * 获取聊天内容。
 * @return 聊天内容
 */
const char* Chat_C2S::getContent() const
{
    return m_szContent;
}

/********** 服务器向客户端发送的聊天信息 **********/
/**
 * 构造方法。
 * @param qsSrcUserName 源用户名
 * @param qsDstUserName 目标用户名
 * @param qsContent     聊天内容
 * @param qsTime        时间
 */
Chat_S2C::Chat_S2C(const QString& qsSrcUserName,
    const QString& qsDstUserName, const QString& qsContent, const QString& qsTime) :
    Chat_C2S(qsSrcUserName, qsDstUserName, qsContent)
{
    strcpy( m_szTime, qsTime.toStdString().c_str() );
}

/**
 * 获取时间。
 * @return 时间
 */
const char* Chat_S2C::getTime() const
{
    return m_szTime;
}

/********** 离线请求 **********/
/**
 * 构造方法。
 * @param qsUserName    用户名
 */
LogoffRequest::LogoffRequest(const QString& qsUserName) :
    Protocol(Protocol::Logoff)
{
    strcpy( m_szUserName, qsUserName.toStdString().c_str() );
}

/**
 * 获取用户名。
 * @return 用户名
 */
const char* LogoffRequest::getUserName() const
{
    return m_szUserName;
}

/********** 好友上线通知 **********/
/**
 * 构造方法。
 * @param szUserName    用户名
 */
FriendLogon::FriendLogon(const char* szUserName) :
    Protocol(Protocol::FriendLogon)
{
    strcpy(m_szUserName, szUserName);
}

/**
 * 获取用户名。
 * @return 用户名
 */
const char* FriendLogon::getUserName() const
{
    return m_szUserName;
}
