#include <QHostAddress>
#include <QMessageBox>
#include "client.h"
#include "logonwindow.h"
#include "regwindow.h"
#include "chatwindow.h"
#include "../Common/const.h"
#include "../Common/config.h"

Config g_config(Const::CONFIG_FILE_NAME);   // 全局配置对象
QString Client::s_qsName;                   // 用户名

/**
 * 构造方法。
 * @param mainWindow    主窗口
 */
Client::Client(QWidget* mainWindow) :
    m_mainWindow(mainWindow),
    m_nFriendsCount(0)
{
    // 连接信号和槽
    QObject::connect( this, SIGNAL(readyRead()), SLOT(onGetMessageFromServer()) );

    // 连接服务器
    QHostAddress serverAddr = g_config.getServerAddr();
    quint16 nServerPort = g_config.getServerPort();
    this->connectToHost(serverAddr, nServerPort);
}

/**
 * 析构方法。
 */
Client::~Client()
{
    // 断开连接
    this->disconnect();

    // 关闭 socket
    this->close();
}

/**
 * 登录。
 * @return 结果
 */
Result::Code Client::logon()
{
    // 显示登录窗口
    LogonWindow logonWindow;
    int nResult = logonWindow.exec();
    if (nResult == QDialog::Rejected)
    {
        return Result::LOGON_SUCCESS;
    }

    // 获取用户输入的用户名和密码
    const QString& qsUserName = logonWindow.getUserName();
    const QString& qsPassword = logonWindow.getPassword();

    // 发送登录请求
    LogonRequest request(qsUserName, qsPassword);
    qint64 nRet = this->write( (const char*)&request, sizeof(LogonRequest) );
    if (nRet == -1)
    {
        return Result::NETWORK_WRONG;
    }

    s_qsName = qsUserName;

    return Result::LOGON_SUCCESS;
}

/**
 * 注册。
 * @return 结果
 */
Result::Code Client::reg()
{
    // 显示注册窗口
    RegWindow regWindow;
    int nResult = regWindow.exec();
    if (nResult == QDialog::Rejected)
    {
        return Result::REG_SUCCESS;
    }

    // 获取用户名、密码和头像图标文件名
    const QString& qsUserName = regWindow.getUserName();
    const QString& qsPassword = regWindow.getPassword();
    int nIconIndex = regWindow.getIconIndex();

    // 发送注册请求
    RegRequest request(qsUserName, qsPassword, nIconIndex);
    qint64 nRet = this->write( (const char*)&request, sizeof(RegRequest) );
    if (nRet == -1)
    {
        return Result::NETWORK_WRONG;
    }

    // 保存用户名和头像索引
    s_qsName = qsUserName;
    m_nIconIndex = nIconIndex;

    return Result::REG_SUCCESS;
}

/**
 * 聊天。
 * @param qsDstUserName 目标用户
 * @param qsContent     聊天内容
 * @return 结果
 */
Result::Code Client::chat(const QString& qsDstUserName, const QString& qsContent)
{
    Chat_C2S chatInfo(s_qsName, qsDstUserName, qsContent);
    qint64 nRet = this->write( (char*)&chatInfo, sizeof(Chat_C2S) );
    if (nRet == -1)
    {
        return Result::NETWORK_WRONG;
    }

    return Result::SUCCESS;
}

/**
 * 添加好友。
 * @param qsFriendName  好友名字
 * @return 结果
 */
Result::Code Client::addFriend(const QString& qsFriendName)
{
    // 检查是否超过好友上限
    if (m_nFriendsCount > Const::MAX_FRIENDS_COUNT)
    {
        return Result::TOO_MANY_FRIENDS;
    }

    // 检查好友名
    if ( qsFriendName.isEmpty() )
    {
        return Result::USER_NAME_EMPTY;
    }
    if ( qsFriendName.contains(Const::SPLIT_FLAT) )
    {
        return Result::USER_NAME_INVALID;
    }
    if ( qsFriendName.length() > Const::MAX_USER_NAME_LEN )
    {
        return Result::USER_NAME_TOO_LONG;
    }
    if (qsFriendName == s_qsName)
    {
        return Result::FRIENDNAME_EQUAL_SELFNAME;
    }

    // 发送添加好友请求
    AddFriendRequest request(s_qsName, qsFriendName, m_nIconIndex);
    qint64 nRet = this->write( (const char*)&request, sizeof(AddFriendRequest) );
    if (nRet == -1)
    {
        return Result::NETWORK_WRONG;
    }

    return Result::SUCCESS;
}

/**
 * 离线。
 * @return 结果
 */
Result::Code Client::logoff()
{
    LogoffRequest request(s_qsName);
    qint64 nRet = this->write( (const char*)&request, sizeof(LogoffRequest) );
    bool bRet = this->flush();
    s_qsName.clear();
    if (nRet == -1 || !bRet)
    {
        return Result::NETWORK_WRONG;
    }

    return Result::SUCCESS;
}

/**
 * 获取自己的名字。
 * @return 自己的名字
 */
const QString& Client::getName()
{
    return s_qsName;
}

/**
 * 服务器消息的响应方法。利用该方法接收服务器发来的信息，并交给对应的方法来处理。
 */
void Client::onGetMessageFromServer()
{
    QByteArray datagram = this->readAll();
    Protocol* protocol = (Protocol*)datagram.data();

    switch ( protocol->getType() )
    {
    case Protocol::Logon:
        this->handleLogonResult( (LogonResult*)protocol );
        break;
    case Protocol::Reg:
        this->handleRegResult( (RegResult*)protocol );
        break;
    case Protocol::Chat:
        this->handleChat( (Chat_S2C*)protocol );
        break;
    case Protocol::AddFriendRequest:
        this->handleAddFriendRequest( (AddFriendRequest*)protocol );
        break;
    case Protocol::AddFriendResult:
        this->handleAddFriendResult( (AddFriendResult*)protocol );
        break;
    case Protocol::Logoff:
        this->handleLogoffRequest( (LogoffRequest*)protocol );
        break;
    case Protocol::FriendLogon:
        this->handleFriendLogon( (FriendLogon*)protocol );
        break;
    default:
        break;
    }
}

/**
 * 处理登录结果。
 * @param result    登录结果
 */
void Client::handleLogonResult(const LogonResult* result)
{
    Result::Code resultCode = result->getResultCode();

    switch (resultCode)
    {
    case Result::LOGON_SUCCESS:
        m_nIconIndex = result->getIconIndex();
        emit UM_LOGON_SUCCESS(result);
        break;
    case Result::USER_NOT_FOUND:
    case Result::PASSWORD_WRONG:
    case Result::USER_ALREADY_LOGON:
        s_qsName.clear();
        QMessageBox::critical( m_mainWindow, NULL, Result::getMessage(resultCode) );
        this->logon();
        break;
    default:
        break;
    }
}

/**
 * 处理注册结果。
 * @param result    注册结果
 */
void Client::handleRegResult(const RegResult* result)
{
    Result::Code resultCode = result->getResultCode();

    switch (resultCode)
    {
    case Result::REG_SUCCESS:
        emit UM_REG_SUCCESS();
        break;
    case Result::USER_ALREADY_LOGON:
    case Result::USER_EXISTS:
        s_qsName.clear();
        QMessageBox::critical( m_mainWindow, NULL, Result::getMessage(resultCode) );
        this->reg();
        break;
    default:
        break;
    }
}

/**
 * 处理聊天。
 * @param chatInfo  聊天信息
 */
void Client::handleChat(const Chat_S2C* chatInfo)
{
    emit UM_CHAT_S2C(chatInfo);
}

/**
 * 处理添加好友请求。
 * @param request   添加好友请求
 */
void Client::handleAddFriendRequest(const AddFriendRequest* request)
{
    const QString qsSrcUserName( request->getSrcUserName() );
    const QString qsDstUserName( request->getDstUserName() );

    QString qsMessage(qsSrcUserName);
    qsMessage += tr("IDS_CONFIRM_ADD_FRIEND");

    int nRet = QMessageBox::question(m_mainWindow, NULL, qsMessage);
    if (nRet == QMessageBox::Yes)   // 同意
    {
        // 向服务器发送结果
        AddFriendResult result(Result::ADD_FRIEND_ALLOW,
            qsSrcUserName, qsDstUserName, m_nIconIndex);
        this->write( (const char*)&result, sizeof(AddFriendResult) );

        // 将对方加为好友
        emit UM_ADD_FRIEND_SUCCESS( qsSrcUserName, request->getIconIndex() );
    }
    else                            // 不同意
    {
        // 向服务器发送结果
        AddFriendResult result(Result::ADD_FRIEND_NOT_ALLOW, qsSrcUserName, qsDstUserName, 0);
        this->write( (const char*)&result, sizeof(AddFriendResult) );
    }
}

/**
 * 处理添加好友结果。
 * @param result    添加好友结果
 */
void Client::handleAddFriendResult(const AddFriendResult* result)
{
    Result::Code resultCode = result->getResultCode();
    const char* pszFriendName = result->getDstUserName();
    QString qsFriendName(pszFriendName);
    QString qsMessage(pszFriendName);

    switch (resultCode)
    {
    case Result::ADD_FRIEND_ALLOW:       // 对方同意
        emit UM_ADD_FRIEND_SUCCESS( qsFriendName, result->getIconIndex() );
        break;
    case Result::USER_NOT_FOUND:         // 该用户尚未注册
    case Result::ALREADY_BE_FRIENDS:     // 已经是你的好友了
    case Result::ADD_FRIEND_NOT_ALLOW:   // 人家不同意
        qsMessage += " ";
        qsMessage += Result::getMessage(resultCode);
        QMessageBox::critical(m_mainWindow, NULL, qsMessage);
        break;
    default:
        break;
    }
}

/**
 * 处理其他用户的离线请求。
 * @param request   其他用户的离线请求
 */
void Client::handleLogoffRequest(const LogoffRequest* request)
{
    emit UM_FRIEND_LOGOFF(request);
}

/**
 * 处理其他用户的登录消息。
 * @param fFriendLogon  其他用户的登录消息
 */
void Client::handleFriendLogon(const FriendLogon* friendLogon)
{
    emit UM_FRIEND_LOGON(friendLogon);
}
