#include <QMessageBox>
#include <QInputDialog>
#include "mainwindow.h"
#include "ui_mainwindow.h"

/**
 * 构造方法。
 * @param parent    父窗口
 */
MainWindow::MainWindow(QWidget* parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    m_client(this),
    m_chatManager(this)
{
    ui->setupUi(this);

    // 连接信号与槽
    QObject::connect( ui->listFriends, SIGNAL(itemDoubleClicked(QListWidgetItem*)),
        SLOT(onItemDoubleClicked(QListWidgetItem*)) );

    QObject::connect( &m_client, SIGNAL(UM_LOGON_SUCCESS(const LogonResult*)),
        SLOT(onLogonSuccess(const LogonResult*)) );

    QObject::connect( &m_client, SIGNAL(UM_REG_SUCCESS()), SLOT(onRegSuccess()) );

    QObject::connect( &m_client, SIGNAL(UM_ADD_FRIEND_SUCCESS(const QString&, int)),
        SLOT(onAddFriendSuccess(const QString&, int)) );

    QObject::connect( &m_client, SIGNAL(UM_CHAT_S2C(const Chat_S2C*)),
        SLOT(onChatS2C(const Chat_S2C*)) );

    QObject::connect( &m_client, SIGNAL(UM_FRIEND_LOGOFF(const LogoffRequest*)),
        SLOT(onFriendLogoff(const LogoffRequest*)) );

    QObject::connect( &m_client, SIGNAL(UM_FRIEND_LOGON(const FriendLogon*)),
        SLOT(onFriendLogon(const FriendLogon*)) );
}

/**
 * 析构方法。
 */
MainWindow::~MainWindow()
{
    // 清空好友列表控件
    ui->listFriends->clear();

    // 离线
    m_client.logoff();

    delete ui;
}

/**
 * 添加一个好友到好友列表控件中。
 * @param qsFriendName  好友名
 * @param nIconIndex    好友头像索引
 * @param bOnline       true 表示好友在线，false 表示好友不在线
 */
void MainWindow::addOneFriend(const QString& qsFriendName, int nIconIndex, bool bOnline)
{
    QString qsIconFilePath = QString(Const::ICON_FILE_NAME).
        arg( nIconIndex, 2, 10, QLatin1Char('0') );
    ui->listFriends->addItem(qsFriendName, qsIconFilePath, bOnline);
}

/**
 * 添加所有好友到好友列表控件中。
 * @param result    登录结果
 */
void MainWindow::addAllFriends(const LogonResult* result)
{
    // 获取好友名列表、图标文件名列表、在线标志
    QStringList friendNames;
    QList<int> friendIconIndexs;
    bool bOnline = false;

    result->getFriendNames(friendNames);
    result->getFriendIconIndexs(friendIconIndexs);

    // 遍历好友名和图标文件夹名列表
    int nCount = result->getFriendsCount();
    for (int i = 0; i < nCount; i++)
    {
        bOnline = result->isFriendOnline(i);
        this->addOneFriend(friendNames[i], friendIconIndexs[i], bOnline);
    }
}

/**
 * “登录”按钮的单击响应方法。
 */
void MainWindow::on_buttonLogon_clicked()
{
    Result::Code resultCode = m_client.logon();
    if (resultCode != Result::LOGON_SUCCESS)
    {
        QMessageBox::critical( this, NULL, Result::getMessage(resultCode) );
    }
}

/**
 * “注册”按钮的单击响应方法。
 */
void MainWindow::on_buttonReg_clicked()
{
    Result::Code resultCode = m_client.reg();
    if (resultCode != Result::REG_SUCCESS)
    {
        QMessageBox::critical( this, NULL, Result::getMessage(resultCode) );
    }
}

/**
 *  “添加好友”按钮的单击响应方法。
 */
void MainWindow::on_buttonAddFriend_clicked()
{
    // 弹出输入框，让用户输入好友名
    QInputDialog inputWindow(this);
    bool bOK = false;
    QString qsDefault;

    QString qsUserName = inputWindow.getText( this, NULL,
        tr("IDS_INUTBOX_TIPTEXT"), QLineEdit::Normal, qsDefault, &bOK );
    if (!bOK)   // bOK 为 false 表示单击了输入框的取消按钮
    {
        return;
    }

    // 执行添加好友
    Result::Code resultCode = m_client.addFriend(qsUserName);
    if (resultCode != Result::SUCCESS)
    {
        QMessageBox::critical( this, NULL, Result::getMessage(resultCode) );
    }
}

/**
 * “群聊”按钮的单击响应方法。
 */
void MainWindow::on_buttonMChat_clicked()
{
    m_chatManager.getChatWindow(Const::MULTI_CHAT)->show();
}

/**
 *  “离线”按钮的单击响应方法。
 */
void MainWindow::on_buttonLogoff_clicked()
{
    // 设置各种控件的状态
    ui->labelIsLogon->setText( tr("IDS_NOT_LOGON") );
    ui->buttonLogon->setEnabled(true);
    ui->buttonReg->setEnabled(true);
    ui->buttonAddFriend->setEnabled(false);
    ui->buttonMChat->setEnabled(false);
    ui->buttonLogoff->setEnabled(false);

    // 清空各种列表
    ui->listFriends->clear();
    m_chatManager.clear();

    // 发离线请求
    m_client.logoff();
}

/**
 * 双击好友列表中某个列表项的响应方法。
 * @param item  被双击的列表项
 */
void MainWindow::onItemDoubleClicked(QListWidgetItem* item)
{
    QString qsFriendName( item->text() );
    ChatWindow* chatWindow = m_chatManager.getChatWindow(qsFriendName);
    chatWindow->show();
}

/**
 * 登录成功事件的响应方法。
 * @param result    登录结果
 */
void MainWindow::onLogonSuccess(const LogonResult* result)
{
    ui->labelIsLogon->setText( tr("IDS_HAS_LOGON") );
    ui->buttonLogon->setEnabled(false);
    ui->buttonReg->setEnabled(false);
    ui->buttonAddFriend->setEnabled(true);
    ui->buttonMChat->setEnabled(true);
    ui->buttonLogoff->setEnabled(true);
    this->addAllFriends(result);
}

/**
 * 注册成功事件的响应方法。
 */
void MainWindow::onRegSuccess()
{
    QMessageBox::information( this, NULL, Result::getMessage(Result::REG_SUCCESS) );
    ui->labelIsLogon->setText( tr("IDS_HAS_LOGON") );
    ui->buttonLogon->setEnabled(false);
    ui->buttonReg->setEnabled(false);
    ui->buttonAddFriend->setEnabled(true);
    ui->buttonMChat->setEnabled(true);
    ui->buttonLogoff->setEnabled(true);
}

/**
 * 聊天事件(客户端到服务器)的响应方法。
 * @param qsName    好友名
 * @param qsContent 聊天内容
 */
void MainWindow::onChatC2S(const QString& qsName, const QString& qsContent)
{
    m_client.chat(qsName, qsContent);
}

/**
 * 聊天事件(服务器到客户端)的响应方法。
 * @param chat  聊天信息
 */
void MainWindow::onChatS2C(const Chat_S2C* chat)
{
    QString qsSrcUserName( chat->getSrcUserName() );
    QString qsDstUserName( chat->getDstUserName() );
    ChatWindow* chatWindow = NULL;

    if (qsDstUserName == Const::MULTI_CHAT)
    {
        chatWindow = m_chatManager.getChatWindow(Const::MULTI_CHAT);
    }
    else
    {
        chatWindow = m_chatManager.getChatWindow(qsSrcUserName);
    }
    chatWindow->setContent(chat);
    chatWindow->show();
}

/**
 * 添加好友成功事件的响应方法。
 * @param qsFriendName  好友名
 * @param nIconIndex    好友头像索引
 */
void MainWindow::onAddFriendSuccess(const QString& qsFriendName, int nIconIndex)
{
    QString qsMessage( qsFriendName + Result::getMessage(Result::ADD_FRIEND_ALLOW) );
    QMessageBox::information(this, NULL, qsMessage);
    this->addOneFriend(qsFriendName, nIconIndex);
}

/**
 * 好友离线事件的响应方法。
 * @param request   离线请求
 */
void MainWindow::onFriendLogoff(const LogoffRequest* request)
{
    ui->listFriends->changeItemColor( request->getUserName(), false );
}

/**
 * 好友登录事件的响应方法。
 * @param friendLogon   好友登录信息
 */
void MainWindow::onFriendLogon(const FriendLogon* friendLogon)
{
    ui->listFriends->changeItemColor( friendLogon->getUserName(), true );
}
