#include <QFile>
#include <QSqlQuery>
#include <QMessageBox>
#include <QNetworkInterface>
#include <QHostInfo>
#include <QDateTime>
#include <QProcess>
#include <string.h>
#include "server.h"
#include "../Common/config.h"
#include "../Common/result.h"

using namespace std;

/**
 * 构造方法。
 */
Server::Server() : m_database( QSqlDatabase::addDatabase(Const::DB_DRIVER) )
{
    // 创建数据库
    this->createDatabase();

    // 连接信号和槽
    QObject::connect( this, SIGNAL(newConnection()), SLOT(onConnect()) );
}

/**
 * 析构方法。
 */
Server::~Server()
{
    // 停止服务
    this->stop();
}

/**
 * 开启服务。
 * @return true 表示成功，false 表示失败
 */
bool Server::start()
{
    // 打开数据库
    m_database.open();

    QHostAddress ip;                    // IP 地址
    QHostAddress broadcast;             // 广播地址
    quint16 nPort = Const::PORT_BASE;   // 端口号
    int nRet = -1;                      // ping 结果

    // 遍历每一个网络接口
    QList<QNetworkInterface> niList = QNetworkInterface::allInterfaces();
    foreach (QNetworkInterface ni, niList) // list 是网络接口列表
    {
        if (nRet == 0)
        {
            break;
        }

        // 遍历该网络接口下的每个入口，找到可用的 IP 地址
        QList<QNetworkAddressEntry> naList = ni.addressEntries();
        foreach (QNetworkAddressEntry na, naList)
        {
            ip = na.ip();
            broadcast = na.broadcast();
            if ( ip.protocol() == QAbstractSocket::IPv4Protocol && !broadcast.isNull() )
            {
#ifdef Q_OS_WIN
                nRet = QProcess::execute( "ping " + broadcast.toString() + " -n 1 -i 20" );
#else
                nRet = QProcess::execute( "ping -b " + broadcast.toString() + " -c 1 -t 20" );
#endif
                if (nRet == 0)  // 找到了
                {
                    // 获取端口号
                    while ( !this->listen(ip, nPort) )
                    {
                        nPort++;
                    }

                    // 将 ip 和 端口号保存到配置文件中
                    Config config(Const::CONFIG_FILE_NAME);
                    config.setServerAddr( ip.toString() );
                    config.setServerPort(nPort);
                    break;
                }
            }
        }
    }

    return nRet == 0;
}

/**
 * 关闭服务。
 */
void Server::stop()
{
    // 关闭数据库
    m_database.close();

    // 关闭服务
    this->close();

    // 清空在线用户列表
    OnlineUsers::iterator end = m_onLineUsers.end();
    for ( OnlineUsers::iterator i = m_onLineUsers.begin(); i != end; i++ )
    {
        delete i.value();
    }
    m_onLineUsers.clear();
}

/**
 * 客户端连接事件的响应方法。
 */
void Server::onConnect()
{
    QTcpSocket* client = this->nextPendingConnection();
    if (client != NULL)
    {
        QObject::connect( client, SIGNAL(readyRead()), SLOT(onGetMessageFromClient()) );
    }
}

/**
 * 接收到客户端发来的消息事件的响应方法。
 */
void Server::onGetMessageFromClient()
{
    QTcpSocket* srcClient = static_cast<QTcpSocket*>( this->sender() ); // 客户端
    QByteArray data = srcClient->readAll();                             // 客户端发送来的数据
    Protocol* protocol = (Protocol*)( data.data() );                    // 协议

    switch ( protocol->getType() )
    {
    case Protocol::Logon:
        this->handleLogon( (LogonRequest*)protocol, srcClient );
        break;
    case Protocol::Reg:
        this->handleReg( (RegRequest*)protocol, srcClient );
        break;
    case Protocol::Chat:
        this->handleChat( (Chat_C2S*)protocol );
        break;
    case Protocol::AddFriendRequest:
        this->handleAddFriendRequest( (AddFriendRequest*)protocol, srcClient );
        break;
    case Protocol::AddFriendResult:
        this->handleAddFriendResult( (AddFriendResult*)protocol );
        break;
    case Protocol::Logoff:
        this->handleLogoff( (LogoffRequest*)protocol );
        break;
    default:
        break;
    }
}

/**
 * 创建数据库。
 */
void Server::createDatabase()
{
    Config config(Const::CONFIG_FILE_NAME);
    const QString qsDatabaseFile( config.getServerDBPath() + Const::DB_NAME );

    bool bRet = QFile::exists(qsDatabaseFile);
    if (!bRet)
    {
        // 创建数据库
        m_database.setDatabaseName(qsDatabaseFile);
        bRet = m_database.open();

        // 创建表
        QSqlQuery query(m_database);
        QString strSQL;

        /* 用户表 */
        strSQL = QString("CREATE TABLE %1(%2 VARCHAR(%3) PRIMARY KEY,%4 VARCHAR(%5),%6 INTEGER)").
            arg(Const::TABLE_USERS).arg(Const::FIELD_NAME).arg(Const::MAX_USER_NAME_LEN).
            arg(Const::FIELD_PASSWORD).arg(Const::MAX_PASSWORD_LEN).arg(Const::FIELD_ICON_INDEX);
        bRet = query.exec(strSQL);

        /* 好友表 */
        strSQL = QString("CREATE TABLE %1(%2 VARCHAR(%3),%4 VARCHAR(%5))").
            arg(Const::TABLE_FRIENDS).arg(Const::FIELD_OWNER).arg(Const::MAX_USER_NAME_LEN).
            arg(Const::FIELD_FRIEND_NAME).arg(Const::MAX_USER_NAME_LEN);
        bRet = query.exec(strSQL);

        /* 离线聊天表 */
        strSQL = QString("CREATE TABLE %1(%2 VARCHAR(%3),%4 VARCHAR(%5),%6 VARCHAR(%7),%8 VARCHAR(%9))").
            arg(Const::TABLE_OFFLINE_CHAT).arg(Const::FIELD_DST_USER).arg(Const::MAX_USER_NAME_LEN).
            arg(Const::FIELD_SRC_USER).arg(Const::MAX_USER_NAME_LEN).arg(Const::FIELD_CONTENT).
            arg(Const::MAX_CONTENT_LEN).arg(Const::FIELD_TIME).arg(Const::TIME_LEN);
        bRet = query.exec(strSQL);

        /* 离线添加好友请求表 */
        strSQL = QString("CREATE TABLE %1(%2 VARCHAR(%3),%4 VARCHAR(%5))").
            arg(Const::TABLE_OFFLINE_ADD_FRIEND_REQUEST).arg(Const::FIELD_DST_USER).
            arg(Const::MAX_USER_NAME_LEN).arg(Const::FIELD_SRC_USER).arg(Const::MAX_USER_NAME_LEN);
        bRet = query.exec(strSQL);

        /* 离线添加好友结果表 */
        strSQL = QString("CREATE TABLE %1(%2 VARCHAR(%3),%4 VARCHAR(%5),%6 INTEGER)").
            arg(Const::TABLE_OFFLINE_ADD_FRIEND_RESULT).arg(Const::FIELD_SRC_USER).
            arg(Const::MAX_USER_NAME_LEN).arg(Const::FIELD_DST_USER).
            arg(Const::MAX_USER_NAME_LEN).arg(Const::FIELD_ADD_FRIEND_RESULT);
        bRet = query.exec(strSQL);
    }
    else
    {
        // 连接数据库
        m_database.setDatabaseName(qsDatabaseFile);
        bRet = m_database.open();
    }
}

/**
 * 处理登录请求。
 * @param request   登录请求的相关信息
 * @param srcUser   发送请求的客户端
 */
void Server::handleLogon(const LogonRequest* request, QTcpSocket* srcUser)
{
    LogonResult logonResult;
    const char* szUserName = request->getUserName();
    const char* szPassword = request->getPassword();
    QSqlQuery query(m_database);
    QString qsSQL;

    // 检查该用户是否已经登录
    OnlineUsers::iterator it = m_onLineUsers.find(szUserName);
    if ( !it.key().isEmpty() )
    {
        logonResult.setErrorCode(Result::USER_ALREADY_LOGON);
        srcUser->write( (const char*)&logonResult, sizeof(LogonResult) );
        return;
    }

    // 从数据库中查找该用户
    qsSQL = QString("SELECT * FROM %1 WHERE %2='%3'").
        arg(Const::TABLE_USERS).arg(Const::FIELD_NAME).arg(szUserName);
    query.exec(qsSQL);
    query.next();

    // 比对用户名
    const QString qsUserName( query.value(Const::FIELD_NAME).toString() );
    if (qsUserName != szUserName)
    {
        logonResult.setErrorCode(Result::USER_NOT_FOUND);
        srcUser->write( (const char*)&logonResult, sizeof(LogonResult) );
        return;
    }

    // 比对密码
    const QString qsPassword( query.value(Const::FIELD_PASSWORD).toString() );
    if (qsPassword != szPassword)
    {
        logonResult.setErrorCode(Result::PASSWORD_WRONG);
        srcUser->write( (const char*)&logonResult, sizeof(LogonResult) );
        return;
    }

    // 登录成功，获取该用户的相关信息
    logonResult.setErrorCode(Result::LOGON_SUCCESS);
    logonResult.setIconIndex( query.value(Const::FIELD_ICON_INDEX).toInt() );

    // 获取该用户的好友列表
    qsSQL = QString("SELECT * FROM %1 WHERE %2='%3'").
        arg(Const::TABLE_FRIENDS).arg(Const::FIELD_OWNER).arg(szUserName);
    query.exec(qsSQL);

    QString qsOneFriendName;
    QString qsAllFriendNames;
    int nAllFriendIconIndexs[Const::MAX_FRIENDS_COUNT];
    QSqlQuery query_temp(m_database);
    bool bOnline = false;
    int nFriendsCount;

    for ( nFriendsCount = 0; query.next(); nFriendsCount++ )
    {
        qsOneFriendName = query.value(Const::FIELD_FRIEND_NAME).toString();
        qsAllFriendNames += qsOneFriendName + Const::SPLIT_FLAT;

        qsSQL = QString("SELECT %1 FROM %2 WHERE %3='%4'").arg(Const::FIELD_ICON_INDEX).
            arg(Const::TABLE_USERS).arg(Const::FIELD_NAME).arg(qsOneFriendName);
        query_temp.exec(qsSQL);
        query_temp.next();

        nAllFriendIconIndexs[nFriendsCount] = query_temp.value(Const::FIELD_ICON_INDEX).toInt();
        OnlineUsers::iterator it = m_onLineUsers.find(qsOneFriendName);
        bOnline = !it.key().isEmpty();
        logonResult.setFriendsOnline(nFriendsCount, bOnline);

        // 如果好友在线，则通知他该用户已经登录了
        if (bOnline)
        {
            FriendLogon friendLogon(szUserName);
            it.value()->write( (const char*)&friendLogon, sizeof(FriendLogon) );
        }
    }

    if ( !qsAllFriendNames.isEmpty() )
    {
        qsAllFriendNames = qsAllFriendNames.left( qsAllFriendNames.length() - 1 );
        logonResult.setFriendNames(qsAllFriendNames);
    }
    if (nFriendsCount > 0)
    {
        logonResult.setFriendIconIndexs(nAllFriendIconIndexs, nFriendsCount);
        logonResult.setFriendsCount(nFriendsCount);
    }

    // 发送登录结果
    srcUser->write( (const char*)&logonResult, sizeof(LogonResult) );

    // 查询是否有和该用户相关的离线添加好友请求，有则发送
    qsSQL = QString("SELECT * FROM %1 WHERE %2='%3'").
        arg(Const::TABLE_OFFLINE_ADD_FRIEND_REQUEST).arg(Const::FIELD_DST_USER).arg(szUserName);
    query.exec(qsSQL);

    const QString qsDstUserName(szUserName);
    while ( query.next() )
    {
        const QString qsSrcUserName( query.value(Const::FIELD_SRC_USER).toString() );

        qsSQL = QString("SELECT %1 FROM %2 WHERE %3='%4'").arg(Const::FIELD_ICON_INDEX).
            arg(Const::TABLE_USERS).arg(Const::FIELD_NAME).arg(qsSrcUserName);
        QSqlQuery query_temp(m_database);
        query_temp.exec(qsSQL);
        query_temp.next();

        AddFriendRequest addFriendRequest( qsSrcUserName,
            qsDstUserName, query_temp.value(Const::FIELD_ICON_INDEX).toInt() );
        srcUser->write( (const char*)&addFriendRequest, sizeof(AddFriendRequest) );
    }

    qsSQL = QString("DELETE FROM %1 WHERE %2='%3'").
        arg(Const::TABLE_OFFLINE_ADD_FRIEND_REQUEST).arg(Const::FIELD_DST_USER).arg(szUserName);
    query.exec(qsSQL);

    // 查询是否有和该用户相关的离线添加好友结果，有则发送
    qsSQL = QString("SELECT * FROM %1 WHERE %2='%3'").
        arg(Const::TABLE_OFFLINE_ADD_FRIEND_RESULT).arg(Const::FIELD_SRC_USER).arg(szUserName);
    query.exec(qsSQL);

    const QString qsSrcUserName(szUserName);
    while ( query.next() )
    {
        const QString qsDstUserName( query.value(Const::FIELD_DST_USER).toString() );

        qsSQL = QString("SELECT %1 FROM %2 WHERE %3='%4'").arg(Const::FIELD_ICON_INDEX).
            arg(Const::TABLE_USERS).arg(Const::FIELD_NAME).arg(qsDstUserName);
        QSqlQuery query_temp(m_database);
        query_temp.exec(qsSQL);
        query_temp.next();
        int nIconIndex = query_temp.value(Const::FIELD_ICON_INDEX).toInt();
        Result::Code nResult = (Result::Code)query.value(Const::FIELD_ADD_FRIEND_RESULT).toInt();

        AddFriendResult addFriendResult(nResult, qsSrcUserName, qsDstUserName, nIconIndex);
        srcUser->write( (const char*)&addFriendResult, sizeof(AddFriendResult) );
    }

    qsSQL = QString("DELETE FROM %1 WHERE %2='%3'").
        arg(Const::TABLE_OFFLINE_ADD_FRIEND_RESULT).arg(Const::FIELD_SRC_USER).arg(szUserName);
    query.exec(qsSQL);

    // 查询是否有和该用户相关的离线聊天消息，有则发送
    qsSQL = QString("SELECT * FROM %1 WHERE %2='%3'").
        arg(Const::TABLE_OFFLINE_CHAT).arg(Const::FIELD_DST_USER).arg(szUserName);
    query.exec(qsSQL);

    while ( query.next() )
    {
        const QString qsSrcUserName( query.value(Const::FIELD_SRC_USER).toString() );
        const QString qsContent( query.value(Const::FIELD_CONTENT).toString() );
        const QString qsTime( query.value(Const::FIELD_TIME).toString() );
        Chat_S2C chatInfo(qsSrcUserName, qsUserName, qsContent, qsTime);
        srcUser->write( (const char*)&chatInfo, sizeof(Chat_S2C) );
    }

    qsSQL = QString("DELETE FROM %1 WHERE %2='%3'").
        arg(Const::TABLE_OFFLINE_CHAT).arg(Const::FIELD_DST_USER).arg(szUserName);
    query.exec(qsSQL);

    // 将该用户加入在线列表
    m_onLineUsers.insert(szUserName, srcUser);
}

/**
 * 处理注册请求。
 * @param request   注册请求的相关信息
 * @param srcUser   发送请求的客户端
 */
void Server::handleReg(const RegRequest* request, QTcpSocket* srcUser)
{
    RegResult regResult;
    const char* szUserName = request->getUserName();
    const char* szPassword = request->getPassword();
    int nIconIndex = request->getIconIndex();
    QSqlQuery query(m_database);
    QString qsSQL;
    bool bRet = false;

    // 检查该用户是否已经登录
    OnlineUsers::iterator it = m_onLineUsers.find(szUserName);
    if ( !it.key().isEmpty() )
    {
        regResult.setErrorCode(Result::USER_ALREADY_LOGON);
        srcUser->write( (const char*)&regResult, sizeof(RegResult) );
        return;
    }

    // 将该用户信息插入到数据库中
    qsSQL = QString("INSERT INTO %1 VALUES('%2','%3','%4')").
        arg(Const::TABLE_USERS).arg(szUserName).arg(szPassword).arg(nIconIndex);
    bRet = query.exec(qsSQL);
    if (!bRet)
    {
        regResult.setErrorCode(Result::USER_EXISTS);
        srcUser->write( (const char*)&regResult, sizeof(RegResult) );
        return;
    }

    // 发送结果
    regResult.setErrorCode(Result::REG_SUCCESS);
    srcUser->write( (const char*)&regResult, sizeof(RegResult) );

    // 将该用户加入到在线列表中
    m_onLineUsers.insert(szUserName, srcUser);
}

/**
 * 处理聊天。
 * @param chat_c2s   聊天信息
 */
void Server::handleChat(const Chat_C2S* chat_c2s)
{
    const char* szSrcUserName = chat_c2s->getSrcUserName();
    const char* szDstUserName = chat_c2s->getDstUserName();
    const char* szContent = chat_c2s->getContent();
    QSqlQuery query(m_database);
    QString qsSQL;

    // 获取当前时间
    const QString qsTime( QDateTime::currentDateTime().
        toString(Const::DATE_TIME_FORMAT) );
    const string strTime( qsTime.toStdString() );
    const char* szTime = strTime.c_str();

    // 处理群聊
    QString qsDstUserName(szDstUserName);
    if (qsDstUserName == Const::MULTI_CHAT)
    {
        Chat_S2C chat(szSrcUserName,
            szDstUserName, szContent, szTime);
        OnlineUsers::iterator end = m_onLineUsers.end();
        for ( OnlineUsers::iterator i = m_onLineUsers.begin(); i != end; i++ )
        {
            if ( i.key() == szSrcUserName )
            {
                continue;
            }
            i.value()->write( (const char*)&chat, sizeof(Chat_S2C) );
        }
        return;
    }

    // 如果目标用户不在线，则将聊天信息插入到数据库中
    OnlineUsers::iterator it = m_onLineUsers.find(szDstUserName);
    if ( it.key().isEmpty() )
    {
        qsSQL = QString("INSERT INTO %1 VALUES('%2','%3','%4','%5')").
            arg(Const::TABLE_OFFLINE_CHAT).arg(szDstUserName).
            arg(szSrcUserName).arg(szContent).arg(qsTime);
        query.exec(qsSQL);
        return;
    }

    // 发送给目标用户
    Chat_S2C chat_s2c(szSrcUserName, szDstUserName, szContent, szTime);
    it.value()->write( (const char*)&chat_s2c, sizeof(Chat_S2C) );
}

/**
 * 处理添加好友请求。
 * @param request       请求的相关信息
 * @param srcUser       发送请求的客户端
 */
void Server::handleAddFriendRequest(const AddFriendRequest* request, QTcpSocket* srcUser)
{
    const char* szSrcUserName = request->getSrcUserName();
    const char* szDstUserName = request->getDstUserName();
    QSqlQuery query(m_database);
    QString qsSQL;

    // 检查目标用户是否存在
    qsSQL = QString("SELECT * FROM %1 WHERE %2='%3'").
        arg(Const::TABLE_USERS).arg(Const::FIELD_NAME).arg(szDstUserName);
    query.exec(qsSQL);
    query.next();
    if ( !query.isValid() )
    {
        // 查无此人，通知源用户
        AddFriendResult result(Result::USER_NOT_FOUND, NULL, szDstUserName, 0);
        srcUser->write( (const char*)&result, sizeof(AddFriendResult) );
        return;
    }

    // 检查是否已经是好友了
    qsSQL = QString("SELECT * FROM %1 WHERE %2='%3' AND %4='%5')").
        arg(Const::TABLE_FRIENDS).arg(Const::FIELD_OWNER).
        arg(szSrcUserName).arg(Const::FIELD_FRIEND_NAME).arg(szDstUserName);
    query.exec(qsSQL);
    query.next();
    if ( query.isValid() )
    {
        AddFriendResult result(Result::ALREADY_BE_FRIENDS, NULL, szDstUserName, 0);
        srcUser->write( (const char*)&result, sizeof(AddFriendResult) );
    }

    // 如果目标用户不在线，则将请求信息插入到数据库中
    OnlineUsers::iterator it = m_onLineUsers.find(szDstUserName);
    if ( it.key().isEmpty() )
    {
        qsSQL = QString("INSERT INTO %1 VALUES('%2','%3')").
            arg(Const::TABLE_OFFLINE_ADD_FRIEND_REQUEST).arg(szDstUserName).arg(szSrcUserName);
        query.exec(qsSQL);
        return;
    }

    // 向目标用户发请求
    it.value()->write( (const char*)request, sizeof(AddFriendRequest) );
}

/**
 * 处理添加好友结果。
 * @param request   结果
 */
void Server::handleAddFriendResult(const AddFriendResult* result)
{
    const char* szSrcUserName = result->getSrcUserName();
    const char* szDstUserName = result->getDstUserName();

    Result::Code errResult = result->getResultCode();
    QSqlQuery query(m_database);
    QString qsSQL;

    // 如果结果是同意，则将相关信息写到好友表中
    if (errResult == Result::ADD_FRIEND_ALLOW)
    {
        qsSQL = QString("INSERT INTO %1 VALUES('%2','%3')").
            arg(Const::TABLE_FRIENDS).arg(szSrcUserName).arg(szDstUserName);
        query.exec(qsSQL);

        qsSQL = QString("INSERT INTO %1 VALUES('%2','%3')").
            arg(Const::TABLE_FRIENDS).arg(szDstUserName).arg(szSrcUserName);
        query.exec(qsSQL);
    }

    // 如果目标用户不在线，则将结果信息插入到数据库中
    OnlineUsers::iterator it = m_onLineUsers.find(szSrcUserName);
    if ( it.key().isEmpty() )
    {
        qsSQL = QString("INSERT INTO %1 VALUES('%2','%3','%4')").
            arg(Const::TABLE_OFFLINE_ADD_FRIEND_RESULT).
            arg(szSrcUserName).arg(szDstUserName).arg(errResult);
        query.exec(qsSQL);
        return;
    }

    // 向目标用户发请求
    it.value()->write( (const char*)result, sizeof(AddFriendResult) );
}

/**
 * 处理离线请求。
 * @param request   离线请求
 */
void Server::handleLogoff(const LogoffRequest* request)
{
	// 广播该用户的离线请求
    OnlineUsers::iterator end = m_onLineUsers.end();
    for ( OnlineUsers::iterator i = m_onLineUsers.begin(); i != end; i++ )
    {
        i.value()->write( (const char*)request, sizeof(LogoffRequest) );
    }

    // 将该用户从在线列表移除
    const char* szUserName = request->getUserName();
    m_onLineUsers.remove(szUserName);
}
